@@ -423,6 +423,7 @@ NETFILE *fopen_net(const char *pathname, const char *mode) {
 		return(NULL);
 	}
 
+	ret->free_buf = 1;
 	ret->bufsize_s = ret->bufsize = 32768;
 	ret->buf_s = ret->buf = malloc(ret->bufsize);
 	ret->bufused = 0;
@@ -611,7 +612,7 @@ int fclose_net(NETFILE *stream) {
 	fd = stream->fd;
 	is_sock = stream->socket;
 
-	if (stream->buf_s) {
+	if (stream->buf_s && stream->free_buf) {
 		free(stream->buf_s);
 	}
 	if (stream->url) {
@@ -638,12 +639,17 @@ int setvbuf_net(NETFILE *stream, char *buf, int mode, size_t size) {
 		return(-1);
 	}
 
+	if (stream->buf_s && stream->free_buf) {
+		free(stream->buf_s);
+	}
+
 	memcpy(buf, stream->buf, stream->bufused);
 
 	stream->buf = buf;
 	stream->buf_s = buf;
 	stream->bufsize = size;
 	stream->bufsize_s = size;
+	stream->free_buf = 0;
 
 	return(0);
 }
